/*
** This file is part of the C Utility Toolkit (CUTK)
** Copyright (C) 2002-2005 Chris Osgood
** http://www.cutk.org
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Lesser General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public
** License along with this library; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
** Please report all bugs and problems to "bugs at cutk.org".
*/

/*
** See dir.h for more information
**
** $Id: dir.c 7 2005-01-02 01:02:22Z chris $
*/

#include "dirent.h"
#include <stdlib.h>

void CleanPath(char* path)
{
#ifdef _WIN32
   static const char good_slashchar = '\\';
   static const char bad_slashchar = '/';
#else /* UNIX-like path */
   static const char good_slashchar = '/';
   static const char bad_slashchar = '\\';
#endif

   char last = 0;
   char* pos;

   for (pos = path; *pos; pos++)
   {
      if (*pos == bad_slashchar)
         *pos = good_slashchar;

      if (*pos == good_slashchar && last == good_slashchar)
         continue;

      last = *pos;
      *path = *pos;
      path++;
   }

   *path = 0;
}

#ifdef _WIN32

size_t strnlen(const char* s, size_t max)
{
   size_t i;
   for (i = 0; s[i] != 0x00 && i < max; i++);
   return i;
}

DIR* opendir(const char* name)
{
   DIR* dir;
   char* path;
   size_t len;

   dir = (DIR*)malloc(sizeof(DIR));
   if (dir == 0)
      return 0;

   dir->once = 0;
   len = strnlen(name, MAX_PATH);
   path = (char*)malloc(len + 3);
   strncpy(path, name, len + 1);
   strcat(path, "\\*");
   CleanPath(path);

   dir->handle = FindFirstFile(path, &dir->data);
   free(path);

   if (dir->handle == INVALID_HANDLE_VALUE)
   {
      free(dir);
      return 0;
   }

   return dir;
}

struct dirent* readdir(DIR* dir)
{
   if (dir->once == 0)
      dir->once = 1;
   else
   {
      BOOL rv = FindNextFile(dir->handle, &dir->data);
      if (rv == 0)
         return 0;
   }

   strncpy(dir->entry.d_name, dir->data.cFileName, MAX_PATH);
   dir->entry.d_name[MAX_PATH - 1] = 0x00;

   return &dir->entry;
}

int closedir(DIR* dir)
{
   if (!FindClose(dir->handle))
      return -1;

   free(dir);

   return 0;
}

#endif /* _WIN32 */
